<style> h1 {
    text-align: center;
    font-size: 24px;
    margin-bottom: 10px;
}

table {
    border-collapse: collapse;
    width: 100%;
}

th, td {
    border: 1px solid black;
    padding: 8px;
    text-align: center;
}

th {
    background-color: #f2f2f2;
}  </style>
<head><title>2023年胡润百富榜</title></head><h1>2023年胡润百富榜</h1>